#include "scorecalculator.h"
#include "gamelogicconstants.h"
#include "gamelogic.h"

ScoreCalculator::ScoreCalculator(QObject *parent) :
    QObject(parent)
{
    qDebug("ScoreCalculator Construct");
    scoreLock = new QMutex();
    scoreTimer = new QTimer(this);
    score = 5000;

    //every 500ms we will refresh the score
    connect(scoreTimer, SIGNAL(timeout()),
            this, SLOT(modifyScore()));
}

ScoreCalculator::~ScoreCalculator(){
    qDebug("SocreCalculator Destruct");
    delete scoreLock;
    delete scoreTimer;
}

qint16 ScoreCalculator::getScore()
{
    return this->score;
}

void ScoreCalculator::modifyScore()
{
    scoreLock->lock();

    //score-calculation logic goes here
    qreal scoreModifier = Accelerometer::instance()->getAccelerometerAccelerationDelta();
    if(scoreModifier > 10)
        score = score - 250;
    else if (scoreModifier > 5 && scoreModifier < 10)
        score = score - 50;
    else
        score = score - 5;

    QMap<QString, qint16> map = Bluetooth::instance()->getDeviceMap();

    evaluateDeviceMapData(map);

    scoreLock->unlock();
}

///
///Starts the score calculation
///
void ScoreCalculator::start()
{
    scoreTimer->start(500);
}

///
///Stops the score calculation
///
void ScoreCalculator::stop()
{
    scoreTimer->stop();
}

///
///Evaluates the device map data and modifys score according to it
///
void ScoreCalculator::evaluateDeviceMapData(QMap<QString, qint16> &deviceMap)
{
    QQueue<QString> waypoints = GameLogic::instance()->getWayPoints();
    if(waypoints.count() == 0) return;
    QMapIterator<QString,qint16> it(deviceMap);
    while(it.hasNext()) {
        it.next();
        if(it.key() == waypoints.front()) {
            if(it.value() > GameLogicConstants::WAYPOINT_DONE_THRESHOLD) {
                score = score + 1000;
                waypoints.pop_front();
                if(waypoints.count() == 0) {
                    stop();
                } else {
                    //no need to do stuff
                }
            } else if(it.value() > GameLogicConstants::WAYPOINT_CLOSE_THRESHOLD) {
                score = score + 5;
            }
        }
    }
}
